--------------------------------------------------------------------------------
-- Copyright  2021								|
-- All Rights Reserved.								|
-- 										|
-- Permission to use, copy, modify, and distribute this software in any way	|
-- is not granted without expressed written agreement. This copyright applies	|
-- to the all code, data and artwork related to this project and all associated	|
-- projects published by the author.						|
--										|
-- More information on copyright is available from sources on-line.		|
-- Use an actual factual one!							|
--------------------------------------------------------------------------------

local addonName, NS = ...

local Lib = NS.FizzleLib

--------------------------- Edit Box -------------------------------------------------
local function EditBox_Init(self)--, data, index)
	local text = self.Data[self.Index]
	if not text then
		text = ""
	end
	self.prevvalue = text
	self:SetText(text)
end

local function EditBox_EnterPressed(self)
	local value
	if self.number then
		value = self:GetNumber()
		if not value then
			value = 0
			self:SetText("0")
		end
	else
		 value = self:GetText()
		 if not value then 
		 	value = "" 
		 end
	end
	self:ClearFocus()
	self.Data[self.Index] = value
	if self.PostUpdate then
		self:PostUpdate()
	end
end

function Lib:CreateEditBox(name, parent, data, index, width, height, hideborders)
	local f = CreateFrame("EditBox", "$parent"..name, parent, "InputBoxTemplate")
	if hideborders then
		f.Background = f:CreateTexture(nil, "BACKGROUND", nil, 1)
		f.Background:SetAllPoints()
		f.Background:SetTexture("Interface\\BUTTONS\\WHITE8X8")
		f.Background:SetVertexColor(0.8, 0.8, 0.8, 0.8)
		f.Left:Hide()
		f.Right:Hide()
		f.Middle:Hide()
--		f.Left:SetAlpha(0)
--		f.Right:SetAlpha(0)
--		f.Middle:SetAlpha(0)
	end
	f.hideborders = hideborders
--	if template then
--		f:SetBackdropColor(0, 0, 0, 0)
--	end 
	f:SetSize(width, height)
	f:SetMaxLetters(1000)
	f:SetAutoFocus(false)
--	f:SetFrameStrata("HIGH")
--	f.Index = index
	Lib.InitData(f, data, index, EditBox_Init)
	f:SetScript("OnEscapePressed", function(self)
		if self.prevvalue then
			self:SetText(self.prevvalue)
		else
			self:SetText("")
		end
		self:ClearFocus()
	end)
	f:SetScript("OnEnterPressed", function(self)
		EditBox_EnterPressed(self)
	end)
	function f:SetBackground(textureleft, textureright, texturemiddle)
		if self.hideborders then
			f.Background:SetTexture(textureleft)
		else
			self.Left:SetTexture(textureleft)
			self.Right:SetTexture(textureright or textureleft)
			self.Middle:SetTexture(texturemiddle or textureleft)
			self.Left:SetAlpha(1)
			self.Right:SetAlpha(1)
			self.Middle:SetAlpha(1)
		end
	end
	function f:SetBackgroundColor(r, g, b, a)
		if self.hideborders then
			f.Background:SetVertexColor(r, g, b, a)
		else
			self.Left:SetVertexColor(r, g, b, a)
			self.Right:SetVertexColor(r, g, b, a)
			self.Middle:SetVertexColor(r, g, b, a)
		end
	end
	Lib:AddLabelFuncs(f)
	return f
end

---------------------- Spin Box ---------------------------------------------------------

local function UpdateSpin(self, val, button)
	local f = self.ScrollUp and self or self:GetParent()
	if button == "RightButton" then
		val = val * f.Multiplier
	end
	f:SetText(f.Data[f.Index] + val)
	EditBox_EnterPressed(f)
--	f.Data[f.Index] = f.Data[f.Index] + val
--	f:InitFunc()
end

function Lib:CreateSpinBox(name, parent, data, index, width, height)
	local f = Lib:CreateEditBox(name, parent, data, index, width, height, true)
	f:SetTextInsets(4, 4, 0, 0)
	f.number = true
	f:SetNumeric()
	f.UpdateValue = 1
	f.Multiplier = 10
	f.ScrollUp = CreateFrame("Button", "$parentScrollUp", f)
	f.ScrollUp:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	f.ScrollUp:SetNormalTexture(Lib.TexturePath.."ScrollUp")
	f.ScrollUp:SetHighlightTexture("Interface\\Buttons\\CheckButtonHilight", "ADD")
	f.ScrollUp:SetPoint("TOPLEFT", f, "TOPRIGHT", 1, 0)
	f.ScrollUp:SetPoint("BOTTOMRIGHT", f, "RIGHT", 13, 0)
	f.ScrollUp:HookScript("OnClick", function(self, button)
		UpdateSpin(self, self:GetParent().UpdateValue, button)
	end)
	f.ScrollDown = CreateFrame("Button", "$parentScrollDown", f)
	f.ScrollDown:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	f.ScrollDown:SetNormalTexture(Lib.TexturePath.."ScrollDown")
	f.ScrollDown:SetHighlightTexture("Interface\\Buttons\\CheckButtonHilight", "ADD")
	f.ScrollDown:SetPoint("TOPLEFT", f, "RIGHT", 1, 0)
	f.ScrollDown:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", 13, 0)
	f.ScrollDown:HookScript("OnClick", function(self, button)
		UpdateSpin(self, -self:GetParent().UpdateValue, button)
	end)
	f:HookScript("OnEnterPressed", function(self)
		self.Data[self.Index] = self:GetNumber()
	end)
	f:HookScript("OnMouseWheel", function(self, delta)
		UpdateSpin(self, delta == 1 and self.UpdateValue or -self.UpdateValue)
	end)
	return f
end
